@@ -1,7 +1,13 @@
-#![allow(missing_docs)]
-use crate::{config::utils, error::ConfigError, Result};
-use serde::{Deserialize, Serialize};
-use std::{collections::HashMap, fs, path::Path};
+use std::{
+    collections::HashMap,
+    fs,
+    path::{Path, PathBuf},
+};
+
+use secrecy::{ExposeSecret, SecretString};
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
+
+use super::{KubeconfigError, LoadDataError};
 
 /// [`Kubeconfig`] represents information on how to connect to a remote Kubernetes cluster
 ///
@@ -13,6 +19,7 @@ use std::{collections::HashMap, fs, path::Path};
 /// [`Config`][crate::Config] is the __intended__ developer interface to help create a [`Client`][crate::Client],
 /// and this will handle the difference between in-cluster deployment and local development.
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct Kubeconfig {
     /// General information to be use for cli interactions
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -33,8 +40,10 @@ pub struct Kubeconfig {
     pub extensions: Option<Vec<NamedExtension>>,
 
     // legacy fields TODO: remove
+    /// Legacy field from TypeMeta
     #[serde(skip_serializing_if = "Option::is_none")]
     pub kind: Option<String>,
+    /// Legacy field from TypeMeta
     #[serde(rename = "apiVersion")]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub api_version: Option<String>,
@@ -42,32 +51,43 @@ pub struct Kubeconfig {
 
 /// Preferences stores extensions for cli.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct Preferences {
+    /// Enable colors
     #[serde(skip_serializing_if = "Option::is_none")]
     pub colors: Option<bool>,
+    /// Extensions holds additional information. This is useful for extenders so that reads and writes don't clobber unknown fields.
     #[serde(skip_serializing_if = "Option::is_none")]
     pub extensions: Option<Vec<NamedExtension>>,
 }
 
 /// NamedExtention associates name with extension.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct NamedExtension {
+    /// Name of extension
     pub name: String,
+    /// Additional information for extenders so that reads and writes don't clobber unknown fields
     pub extension: serde_json::Value,
 }
 
 /// NamedCluster associates name with cluster.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct NamedCluster {
+    /// Name of cluster
     pub name: String,
+    /// Information about how to communicate with a kubernetes cluster
     pub cluster: Cluster,
 }
 
 /// Cluster stores information to connect Kubernetes cluster.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct Cluster {
     /// The address of the kubernetes cluster (https://hostname:port).
     pub server: String,
+    /// Skips the validity check for the server's certificate. This will make your HTTPS connections insecure.
     #[serde(rename = "insecure-skip-tls-verify")]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub insecure_skip_tls_verify: Option<bool>,
@@ -90,12 +110,35 @@ pub struct Cluster {
 
 /// NamedAuthInfo associates name with authentication.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct NamedAuthInfo {
+    /// Name of the user
     pub name: String,
+    /// Information that describes identity of the user
     #[serde(rename = "user")]
     pub auth_info: AuthInfo,
 }
 
+fn serialize_secretstring<S>(pw: &Option<SecretString>, serializer: S) -> Result<S::Ok, S::Error>
+where
+    S: Serializer,
+{
+    match pw {
+        Some(secret) => serializer.serialize_str(secret.expose_secret()),
+        None => serializer.serialize_none(),
+    }
+}
+
+fn deserialize_secretstring<'de, D>(deserializer: D) -> Result<Option<SecretString>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    match String::deserialize(deserializer) {
+        Ok(secret) => Ok(Some(SecretString::new(secret))),
+        Err(e) => Err(e),
+    }
+}
+
 /// AuthInfo stores information to tell cluster who you are.
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
 pub struct AuthInfo {
@@ -103,12 +146,20 @@ pub struct AuthInfo {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub username: Option<String>,
     /// The password for basic authentication to the kubernetes cluster.
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub password: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none", default)]
+    #[serde(
+        serialize_with = "serialize_secretstring",
+        deserialize_with = "deserialize_secretstring"
+    )]
+    pub password: Option<SecretString>,
 
     /// The bearer token for authentication to the kubernetes cluster.
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub token: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none", default)]
+    #[serde(
+        serialize_with = "serialize_secretstring",
+        deserialize_with = "deserialize_secretstring"
+    )]
+    pub token: Option<SecretString>,
     /// Pointer to a file that contains a bearer token (as described above). If both `token` and token_file` are present, `token` takes precedence.
     #[serde(rename = "tokenFile")]
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -129,8 +180,12 @@ pub struct AuthInfo {
     pub client_key: Option<String>,
     /// PEM-encoded data from a client key file for TLS. Overrides `client_key`
     #[serde(rename = "client-key-data")]
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub client_key_data: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none", default)]
+    #[serde(
+        serialize_with = "serialize_secretstring",
+        deserialize_with = "deserialize_secretstring"
+    )]
+    pub client_key_data: Option<SecretString>,
 
     /// The username to act-as.
     #[serde(rename = "as")]
@@ -151,15 +206,26 @@ pub struct AuthInfo {
     pub exec: Option<ExecConfig>,
 }
 
+#[cfg(test)]
+impl PartialEq for AuthInfo {
+    fn eq(&self, other: &Self) -> bool {
+        serde_json::to_value(&self).unwrap() == serde_json::to_value(&other).unwrap()
+    }
+}
+
 /// AuthProviderConfig stores auth for specified cloud provider.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct AuthProviderConfig {
+    /// Name of the auth provider
     pub name: String,
+    /// Auth provider configuration
     pub config: HashMap<String, String>,
 }
 
 /// ExecConfig stores credential-plugin configuration.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct ExecConfig {
     /// Preferred input version of the ExecInfo.
     ///
@@ -181,13 +247,17 @@ pub struct ExecConfig {
 
 /// NamedContext associates name with context.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct NamedContext {
+    /// Name of the context
     pub name: String,
+    /// Associations for the context
     pub context: Context,
 }
 
 /// Context stores tuple of cluster and user information.
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(test, derive(PartialEq))]
 pub struct Context {
     /// Name of the cluster for this context
     pub cluster: String,
@@ -206,22 +276,13 @@ const KUBECONFIG: &str = "KUBECONFIG";
 /// Some helpers on the raw Config object are exposed for people needing to parse it
 impl Kubeconfig {
     /// Read a Config from an arbitrary location
-    pub fn read_from<P: AsRef<Path>>(path: P) -> Result<Kubeconfig> {
-        let data = fs::read_to_string(&path).map_err(|source| ConfigError::ReadFile {
-            path: path.as_ref().into(),
-            source,
-        })?;
-        // support multiple documents
-        let mut documents: Vec<Kubeconfig> = vec![];
-        for doc in serde_yaml::Deserializer::from_str(&data) {
-            let value = serde_yaml::Value::deserialize(doc).map_err(ConfigError::ParseYaml)?;
-            let kconf = serde_yaml::from_value(value).map_err(ConfigError::ParseYaml)?;
-            documents.push(kconf)
-        }
+    pub fn read_from<P: AsRef<Path>>(path: P) -> Result<Kubeconfig, KubeconfigError> {
+        let data = fs::read_to_string(&path)
+            .map_err(|source| KubeconfigError::ReadConfig(source, path.as_ref().into()))?;
 
         // Remap all files we read to absolute paths.
         let mut merged_docs = None;
-        for mut config in documents {
+        for mut config in kubeconfig_from_yaml(&data)? {
             if let Some(dir) = path.as_ref().parent() {
                 for named in config.clusters.iter_mut() {
                     if let Some(path) = &named.cluster.certificate_authority {
@@ -254,18 +315,25 @@ impl Kubeconfig {
                 merged_docs = Some(config);
             }
         }
-        let config = merged_docs.ok_or_else(|| ConfigError::EmptyKubeconfig(path.as_ref().to_path_buf()))?;
-        Ok(config)
+        // Empty file defaults to an empty Kubeconfig
+        Ok(merged_docs.unwrap_or_default())
+    }
+
+    /// Read a Config from an arbitrary YAML string
+    ///
+    /// This is preferable to using serde_yaml::from_str() because it will correctly
+    /// parse multi-document YAML text and merge them into a single `Kubeconfig`
+    pub fn from_yaml(text: &str) -> Result<Kubeconfig, KubeconfigError> {
+        kubeconfig_from_yaml(text)?
+            .into_iter()
+            .try_fold(Kubeconfig::default(), Kubeconfig::merge)
     }
 
     /// Read a Config from `KUBECONFIG` or the the default location.
-    pub fn read() -> Result<Kubeconfig> {
+    pub fn read() -> Result<Kubeconfig, KubeconfigError> {
         match Self::from_env()? {
             Some(config) => Ok(config),
-            None => {
-                let path = utils::default_kube_path().ok_or(ConfigError::NoKubeconfigPath)?;
-                Self::read_from(path)
-            }
+            None => Self::read_from(default_kube_path().ok_or(KubeconfigError::FindPath)?),
         }
     }
 
@@ -275,7 +343,7 @@ impl Kubeconfig {
     /// # Panics
     ///
     /// Panics if `KUBECONFIG` value contains the NUL character.
-    pub fn from_env() -> Result<Option<Self>> {
+    pub fn from_env() -> Result<Option<Self>, KubeconfigError> {
         match std::env::var_os(KUBECONFIG) {
             Some(value) => {
                 let paths = std::env::split_paths(&value)
@@ -307,12 +375,12 @@ impl Kubeconfig {
     /// >   Example: Preserve the context of the first file to set `current-context`.
     /// >   Example: If two files specify a `red-user`, use only values from the first file's `red-user`.
     /// >            Even if the second file has non-conflicting entries under `red-user`, discard them.
-    fn merge(mut self, next: Kubeconfig) -> Result<Self> {
+    fn merge(mut self, next: Kubeconfig) -> Result<Self, KubeconfigError> {
         if self.kind.is_some() && next.kind.is_some() && self.kind != next.kind {
-            return Err(ConfigError::KindMismatch.into());
+            return Err(KubeconfigError::KindMismatch);
         }
         if self.api_version.is_some() && next.api_version.is_some() && self.api_version != next.api_version {
-            return Err(ConfigError::ApiVersionMismatch.into());
+            return Err(KubeconfigError::ApiVersionMismatch);
         }
 
         self.kind = self.kind.or(next.kind);
@@ -327,6 +395,16 @@ impl Kubeconfig {
     }
 }
 
+fn kubeconfig_from_yaml(text: &str) -> Result<Vec<Kubeconfig>, KubeconfigError> {
+    let mut documents = vec![];
+    for doc in serde_yaml::Deserializer::from_str(text) {
+        let value = serde_yaml::Value::deserialize(doc).map_err(KubeconfigError::Parse)?;
+        let kubeconfig = serde_yaml::from_value(value).map_err(KubeconfigError::InvalidStructure)?;
+        documents.push(kubeconfig);
+    }
+    Ok(documents)
+}
+
 #[allow(clippy::redundant_closure)]
 fn append_new_named<T, F>(base: &mut Vec<T>, next: Vec<T>, f: F)
 where
@@ -351,30 +429,89 @@ fn to_absolute(dir: &Path, file: &str) -> Option<String> {
 }
 
 impl Cluster {
-    pub(crate) fn load_certificate_authority(&self) -> Result<Option<Vec<u8>>> {
+    pub(crate) fn load_certificate_authority(&self) -> Result<Option<Vec<u8>>, KubeconfigError> {
         if self.certificate_authority.is_none() && self.certificate_authority_data.is_none() {
             return Ok(None);
         }
-        let res =
-            utils::data_or_file_with_base64(&self.certificate_authority_data, &self.certificate_authority)?;
-        Ok(Some(res))
+
+        let ca = load_from_base64_or_file(
+            &self.certificate_authority_data.as_deref(),
+            &self.certificate_authority,
+        )
+        .map_err(KubeconfigError::LoadCertificateAuthority)?;
+        Ok(Some(ca))
     }
 }
 
 impl AuthInfo {
-    pub(crate) fn load_client_certificate(&self) -> Result<Vec<u8>> {
-        utils::data_or_file_with_base64(&self.client_certificate_data, &self.client_certificate)
+    pub(crate) fn identity_pem(&self) -> Result<Vec<u8>, KubeconfigError> {
+        let client_cert = &self.load_client_certificate()?;
+        let client_key = &self.load_client_key()?;
+        let mut buffer = client_key.clone();
+        buffer.extend_from_slice(client_cert);
+        Ok(buffer)
     }
 
-    pub(crate) fn load_client_key(&self) -> Result<Vec<u8>> {
-        utils::data_or_file_with_base64(&self.client_key_data, &self.client_key)
+    pub(crate) fn load_client_certificate(&self) -> Result<Vec<u8>, KubeconfigError> {
+        // TODO Shouldn't error when `self.client_certificate_data.is_none() && self.client_certificate.is_none()`
+
+        load_from_base64_or_file(&self.client_certificate_data.as_deref(), &self.client_certificate)
+            .map_err(KubeconfigError::LoadClientCertificate)
     }
+
+    pub(crate) fn load_client_key(&self) -> Result<Vec<u8>, KubeconfigError> {
+        // TODO Shouldn't error when `self.client_key_data.is_none() && self.client_key.is_none()`
+
+        load_from_base64_or_file(
+            &self
+                .client_key_data
+                .as_ref()
+                .map(|secret| secret.expose_secret().as_str()),
+            &self.client_key,
+        )
+        .map_err(KubeconfigError::LoadClientKey)
+    }
+}
+
+fn load_from_base64_or_file<P: AsRef<Path>>(
+    value: &Option<&str>,
+    file: &Option<P>,
+) -> Result<Vec<u8>, LoadDataError> {
+    let data = value
+        .map(load_from_base64)
+        .or_else(|| file.as_ref().map(load_from_file))
+        .unwrap_or(Err(LoadDataError::NoBase64DataOrFile))?;
+    Ok(ensure_trailing_newline(data))
+}
+
+fn load_from_base64(value: &str) -> Result<Vec<u8>, LoadDataError> {
+    base64::decode(&value).map_err(LoadDataError::DecodeBase64)
+}
+
+fn load_from_file<P: AsRef<Path>>(file: &P) -> Result<Vec<u8>, LoadDataError> {
+    fs::read(&file).map_err(|source| LoadDataError::ReadFile(source, file.as_ref().into()))
+}
+
+// Ensure there is a trailing newline in the blob
+// Don't bother if the blob is empty
+fn ensure_trailing_newline(mut data: Vec<u8>) -> Vec<u8> {
+    if data.last().map(|end| *end != b'\n').unwrap_or(false) {
+        data.push(b'\n');
+    }
+    data
+}
+
+/// Returns kubeconfig path from `$HOME/.kube/config`.
+fn default_kube_path() -> Option<PathBuf> {
+    use dirs::home_dir;
+    home_dir().map(|h| h.join(".kube").join("config"))
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
     use serde_json::Value;
+    use std::str::FromStr;
 
     #[test]
     fn kubeconfig_merge() {
@@ -383,7 +520,7 @@ mod tests {
             auth_infos: vec![NamedAuthInfo {
                 name: "red-user".into(),
                 auth_info: AuthInfo {
-                    token: Some("first-token".into()),
+                    token: Some(SecretString::from_str("first-token").unwrap()),
                     ..Default::default()
                 },
             }],
@@ -395,7 +532,7 @@ mod tests {
                 NamedAuthInfo {
                     name: "red-user".into(),
                     auth_info: AuthInfo {
-                        token: Some("second-token".into()),
+                        token: Some(SecretString::from_str("second-token").unwrap()),
                         username: Some("red-user".into()),
                         ..Default::default()
                     },
@@ -403,7 +540,7 @@ mod tests {
                 NamedAuthInfo {
                     name: "green-user".into(),
                     auth_info: AuthInfo {
-                        token: Some("new-token".into()),
+                        token: Some(SecretString::from_str("new-token").unwrap()),
                         ..Default::default()
                     },
                 },
@@ -416,7 +553,14 @@ mod tests {
         assert_eq!(merged.current_context, Some("default".into()));
         // Auth info with the same name does not overwrite
         assert_eq!(merged.auth_infos[0].name, "red-user".to_owned());
-        assert_eq!(merged.auth_infos[0].auth_info.token, Some("first-token".into()));
+        assert_eq!(
+            merged.auth_infos[0]
+                .auth_info
+                .token
+                .as_ref()
+                .map(|t| t.expose_secret().to_string()),
+            Some("first-token".to_string())
+        );
         // Even if it's not conflicting
         assert_eq!(merged.auth_infos[0].auth_info.username, None);
         // New named auth info is appended
@@ -480,9 +624,7 @@ users:
     client-certificate: /home/kevin/.minikube/profiles/minikube/client.crt
     client-key: /home/kevin/.minikube/profiles/minikube/client.key";
 
-        let config: Kubeconfig = serde_yaml::from_str(config_yaml)
-            .map_err(ConfigError::ParseYaml)
-            .unwrap();
+        let config = Kubeconfig::from_yaml(config_yaml).unwrap();
 
         assert_eq!(config.clusters[0].name, "eks");
         assert_eq!(config.clusters[1].name, "minikube");
@@ -495,7 +637,7 @@ users:
     }
 
     #[test]
-    fn kubeconfig_multi_document_merge() -> Result<()> {
+    fn kubeconfig_multi_document_merge() -> Result<(), KubeconfigError> {
         let config_yaml = r#"---
 apiVersion: v1
 clusters:
@@ -537,9 +679,7 @@ users:
     client-certificate-data: aGVsbG8K
     client-key-data: aGVsbG8K
 "#;
-        let file = tempfile::NamedTempFile::new().expect("create config tempfile");
-        fs::write(file.path(), config_yaml).unwrap();
-        let cfg = Kubeconfig::read_from(file.path())?;
+        let cfg = Kubeconfig::from_yaml(config_yaml)?;
 
         // Ensure we have data from both documents:
         assert_eq!(cfg.clusters[0].name, "k3d-promstack");
@@ -547,4 +687,33 @@ users:
 
         Ok(())
     }
+
+    #[test]
+    fn kubeconfig_from_empty_string() {
+        let cfg = Kubeconfig::from_yaml("").unwrap();
+
+        assert_eq!(cfg, Kubeconfig::default());
+    }
+
+    #[test]
+    fn authinfo_debug_does_not_output_password() {
+        let authinfo_yaml = r#"
+username: user
+password: kube_rs
+"#;
+        let authinfo: AuthInfo = serde_yaml::from_str(authinfo_yaml).unwrap();
+        let authinfo_debug_output = format!("{:?}", authinfo);
+        let expected_output = "AuthInfo { \
+        username: Some(\"user\"), \
+        password: Some(Secret([REDACTED alloc::string::String])), \
+        token: None, token_file: None, client_certificate: None, \
+        client_certificate_data: None, client_key: None, \
+        client_key_data: None, impersonate: None, \
+        impersonate_groups: None, \
+        auth_provider: None, \
+        exec: None \
+        }";
+
+        assert_eq!(authinfo_debug_output, expected_output)
+    }
 }
